/////////////////////////////////////////////////////////////////
//                                                             //
// Tento zdrojov kd je soust distribuce balku program,  //
//     poskytovanch jako doplujc informace ke knize        //
//                                                             //
//                 Java -- bohatstv knihoven                  //
//                                                             //
//     Pette si, prosm, dkladn upozornn v souboru      // 
//                       CTI_ME.TXT                            //
//        kter je nedlnou soust tto distribuce           //
//                                                             //
//                 (c) Pavel Herout, 2003                      // 
//                                                             //
/////////////////////////////////////////////////////////////////


import java.util.*;
import java.io.*;
import java.text.*;

class OsobaProKolekce implements Comparable {
  // zakladni stavove atributy
  // nemenitelne
  private final String  krestni, prijmeni;
  // menitelne
  private int vyska;
  private double vaha;
  // odvozeny atribut - bitovy obraz vahy pro hashCode() a equals()
  protected long longVaha;
  // pomocny atribut pro ceske razeni
  private static final Collator COL = Collator.getInstance(new Locale("cs", "CZ"));

  public OsobaProKolekce(String krestni, String prijmeni, int vyska, double vaha) {
    if (krestni == null  ||  prijmeni == null)
      throw new NullPointerException();
    this.krestni = krestni;
    this.prijmeni = prijmeni;
    setVyska(vyska);
    setVaha(vaha);
  }

  public String getKrestni() {
    return krestni;
  }

  public String getPrijmeni() {
    return prijmeni;
  }

  public void setVyska(int vyska) {
    if (vyska <= 0)
      throw new IllegalArgumentException("vyska=" + vyska);
    this.vyska = vyska;
  }

  public void setVaha(double vaha) {
    if (vaha <= 0)
      throw new IllegalArgumentException("vaha=" + vaha);
    this.vaha = vaha;
    this.longVaha = Double.doubleToLongBits(this.vaha);
  }

  public int getVyska() {
    return vyska;
  }

  public double getVaha() {
    return vaha;
  }

  public String toString() {
    return prijmeni + " " + krestni + ", " + vyska + ", " + vaha + "\n";
  }

  public boolean equals(Object o) {
    if (o == this)
      return true;
    if ((o instanceof OsobaProKolekce) == false)
      return false;
    OsobaProKolekce opk = (OsobaProKolekce) o;
    boolean stejneKrestni = krestni.equals(opk.krestni);
    boolean stejnePrijmeni = prijmeni.equals(opk.prijmeni);
    boolean stejnaVyska = vyska == opk.vyska;
    boolean stejnaVaha = longVaha == opk.longVaha;

    return stejneKrestni && stejnePrijmeni && stejnaVyska && stejnaVaha;
  }

  public int hashCode() {
    int vysledek = 17;
    vysledek = 37 * vysledek + krestni.hashCode();
    vysledek = 37 * vysledek + prijmeni.hashCode();
    vysledek = 37 * vysledek + vyska;
    int pom = (int) (longVaha ^ (longVaha >>> 32));
    vysledek = 37 * vysledek + pom;
    return vysledek;
  }

  // prirozene razeni
  public int compareTo(Object o) {
    OsobaProKolekce opk = (OsobaProKolekce) o;
    int tmpP = COL.compare(this.prijmeni, opk.prijmeni);
    int tmpK = COL.compare(this.krestni, opk.krestni);
    return (tmpP == 0 ? tmpK : tmpP);
  }

  // komparatory pro absolutni razeni
  public static final Comparator PODLE_VYSKY = new Comparator() {
    public int compare(Object o1, Object o2) {
      return ((OsobaProKolekce) o1).vyska - ((OsobaProKolekce) o2).vyska;
    }
  };

  public static final Comparator PODLE_VAHY = new Comparator() {
    public int compare(Object o1, Object o2) {
      double v1 = ((OsobaProKolekce) o1).vaha;
      double v2 = ((OsobaProKolekce) o2).vaha;
      long  lv1 = ((OsobaProKolekce) o1).longVaha;
      long  lv2 = ((OsobaProKolekce) o2).longVaha;
      if (lv1 == lv2)
        return 0;
      if (v1 > v2)
        return +1;
      else
        return -1;
//      return (int) (((OsobaProKolekce) o1).vaha - ((OsobaProKolekce) o2).vaha);
    }
  };

  public static final Comparator PODLE_JMENA = new Comparator() {
    public int compare(Object o1, Object o2) {
      OsobaProKolekce opk1 = (OsobaProKolekce) o1;
      OsobaProKolekce opk2 = (OsobaProKolekce) o2;
      int tmpP = COL.compare(opk1.prijmeni, opk2.prijmeni);
      int tmpK = COL.compare(opk1.krestni,  opk2.krestni);
      return (tmpP == 0 ? tmpK : tmpP);
    }
  };
}

public class TestOsobaProKolekce {
  public static void main(String[] argv) throws IOException {
    OutputStreamWriter o = new OutputStreamWriter(System.out, "Cp852");
    PrintWriter p = new PrintWriter(o);

    ArrayList a = new ArrayList();
    a.add(new OsobaProKolekce("Karel", "Chytr", 160, 60.));
    a.add(new OsobaProKolekce("Karel", "tvrt", 170, 70.));
    a.add(new OsobaProKolekce("Karel", "Ctvrt", 180, 80.));
    a.add(new OsobaProKolekce("Josef", "tvrt", 190, 65.));
    a.add(new OsobaProKolekce("Krel", "tvrt", 200, 90.));
    p.println(a);

    Collections.sort(a);
    p.println("Podle jmna\n" + a);
    Collections.sort(a, OsobaProKolekce.PODLE_VYSKY);
    p.println("Podle vky\n" + a);
    Collections.sort(a, OsobaProKolekce.PODLE_VAHY);
    p.println("Podle vhy\n" + a);
    Collections.sort(a, OsobaProKolekce.PODLE_JMENA);
    p.println("Podle jmna\n" + a);
    p.flush();

    a.add(new OsobaProKolekce("Jan", "Zporn", 0, -5.));
  }
}


